#include "taskFlyport.h"

//DEFINE GPIO SHT75
#define DATA_LINE         p19
#define CLOCK_LINE        p21
#define DATA_LINE_IN      IOInit(DATA_LINE,inup)
#define DATA_LINE_OUT     IOInit(DATA_LINE,0)
#define DATA_LINE_LOW     DATA_LINE_OUT;IOPut(DATA_LINE,0)
#define DATA_LINE_HIGH    DATA_LINE_OUT;IOPut(DATA_LINE,1) 
#define DATA_LINE_READ    IOGet(DATA_LINE)
#define CLOCK_LINE_OUT    IOInit(CLOCK_LINE,0)
#define CLOCK_LINE_LOW    IOPut(CLOCK_LINE,0)
#define CLOCK_LINE_HIGH   IOPut(CLOCK_LINE,1)
#define SHT_TIMEOUT       1
#define CMD_READ_TEMP     0x03
#define CMD_READ_HUM      0x05
#define CMD_RESET         0x1e

//PROTOTIPI
void    SHT_SendStart(void) ;
void    SHT_SendReset (void);
int     SHT_SendByte(char byte);
char    SHT_ReadByte(int withack);
double  SHT_ReadTemperature(void);
double  SHT_ReadHumidity(void);


void SHT_SendStart(void) {
  DATA_LINE_OUT;
  CLOCK_LINE_HIGH;
  vTaskDelay(1);
  DATA_LINE_IN;
  DATA_LINE_LOW;
  CLOCK_LINE_LOW;
  CLOCK_LINE_HIGH;
  vTaskDelay(1);
  DATA_LINE_HIGH;
  CLOCK_LINE_LOW;
  DATA_LINE_IN;
}
void SHT_SendReset (void) {
  int i;
  CLOCK_LINE_OUT;
  for (i=0;i<12;i++) {
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    CLOCK_LINE_LOW;
    vTaskDelay(1);
  }
}
int SHT_SendByte(char byte) {
  char tempbyte;
  int i;
  DWORD t0;
  DATA_LINE_OUT;
  tempbyte=byte;
  for (i=0x80;i>0;i/=2) {
    if (tempbyte & i) {
      DATA_LINE_HIGH;
    } else {
      DATA_LINE_LOW;
    }
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    CLOCK_LINE_LOW;
  }
  DATA_LINE_IN;
  CLOCK_LINE_HIGH;
  t0=TickGet();
  while (DATA_LINE_READ) {
    if((TickGet()-t0)> SHT_TIMEOUT*TICK_SECOND) {
      SHT_SendReset();
      return 0;
    }
  }
  CLOCK_LINE_LOW;
  return 1;
}

char SHT_ReadByte(int withack) {
  char tempbyte;
  int i;
  tempbyte=0;
  DATA_LINE_IN;
  for (i=0x80;i>0;i/=2) {
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    if (DATA_LINE_READ) tempbyte |= i;
    CLOCK_LINE_LOW;
  }  
  vTaskDelay(1);
  if (withack) {
    DATA_LINE_OUT;
    DATA_LINE_LOW;
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    CLOCK_LINE_LOW;
    DATA_LINE_IN;
  } else {
    DATA_LINE_OUT;
    DATA_LINE_HIGH;
    CLOCK_LINE_HIGH;
    vTaskDelay(1);
    CLOCK_LINE_LOW;
    DATA_LINE_IN;
  }
  return tempbyte;
}

double SHT_ReadTemperature(void) {
  char data[2];
  char chk;
  unsigned int itemp;
  DWORD t0;
  SHT_SendStart();
  if (!SHT_SendByte(CMD_READ_TEMP)) return -101;
  t0=TickGet();
  while (DATA_LINE_READ) {
    if((TickGet()-t0)> SHT_TIMEOUT*TICK_SECOND) {
      SHT_SendReset();
      return -101;
    }
  }
  data[0]=(char)SHT_ReadByte(1);
  data[1]=(char)SHT_ReadByte(1);
  chk=SHT_ReadByte(0);
  itemp=((int)data[0]<<8)+((unsigned char)data[1] & 0xFF);
  return ((0.01*((float)itemp))-40.00);
}

double SHT_ReadHumidity(void) { 
  char data[2];
  char chk;
  int itemp;
  DWORD t0;
  SHT_SendStart();
  if (!SHT_SendByte(CMD_READ_HUM)) return -101;
  t0=TickGet();
  while (DATA_LINE_READ) {
    if((TickGet()-t0)> SHT_TIMEOUT*TICK_SECOND) {
      SHT_SendReset();
      return -101;
    }
  }
  data[0]=(char)SHT_ReadByte(1);
  data[1]=(char)SHT_ReadByte(1);
  chk=SHT_ReadByte(0);
  itemp=((int)data[0]<<8)+((unsigned char)data[1] & 0xFF);
  return ((-4)+(0.0405*(float)itemp)+(-0.0000028*((float)itemp*(float)itemp)));
}


void FlyportTask() {
  double       temperature;
  double       humidity;
  char         debug[255];
  DWORD        t0;
  
  temperature   = 0;
  humidity      = 0;
  t0            = 0;
  
  SHT_SendReset();
  UARTWrite(1,"SxFLY PRO test SHT75\r\n");
  while(1) {
    if((TickGet()-t0)>(30*TICK_SECOND)){
      temperature= SHT_ReadTemperature();
      humidity   = SHT_ReadHumidity();
      if(temperature!=-101 && humidity!=-101){ 
        sprintf(debug, "\r\nTemperatura:\t%.02f C\r\nUmidita':\t\t%.02f %%\r\n",temperature, humidity);
        UARTWrite(1,debug);
        t0=TickGet();
      } else {
        UARTWrite(1,"ERRORE di comunicazione con la sonda SHT75\r\n");  
      }
    }
  }
}
